# -*- coding: utf-8 -*-
"""
Created on Tue Feb 26 15:50:57 2019

@author: REDACTED FOR BLIND PEER REVIEW
"""
import Tkinter, tkFileDialog, tkMessageBox
from Tkinter import *
import os
from time import strftime
import numpy as np
import pandas as pd
import matplotlib.pyplot as plt
import matplotlib.dates as md
from matplotlib.ticker import MaxNLocator
import seaborn as sns
sns.set()
import math
# ignore future warnings
import warnings
warnings.filterwarnings("ignore")



# options for drop down menus in dialog window
hours = [1,2,3,4,5,6,7,8,9,10,11,12]
am_pm = ["am","pm"]
my_groups_default = "pairfed, HFD, HFD-chow, chow"

###################################################################
# GUI RELATED PART
###################################################################
class FedApp(Toplevel):

    def __init__(self, parent, title = None):

        Toplevel.__init__(self, parent)
        self.transient(parent)

        if title:
            self.title(title)            

        self.parent = parent
        self.retrieved_id_ints = None
        
        # instantiate window with options and make it in focus
        body = Frame(self)
        self.initial_focus = self.body(body)
        body.pack(padx=5, pady=5)

        # instantaite buttons ok and cancel
        self.buttonbox()

        self.grab_set()

        if not self.initial_focus:
            self.initial_focus = self

        self.protocol("WM_DELETE_WINDOW", self.cancel)

        self.geometry("+%d+%d" % (parent.winfo_rootx()+50,
                                  parent.winfo_rooty()+50))

        self.initial_focus.focus_set()

        self.wait_window(self)


    def body(self, master):
        # dialog body.  return widget with options that should have
        # initial focus.  

        # Labels
        Label(master, text="Select folder with csv files:").grid(row=0,
                                                         columnspan=4,
                                                         sticky=W,
                                                         padx=5,
                                                         pady=5)
        self.select_folder_btn = Button(master, text="Select", command=self.show_folders)
        Label(master, text="Lights out at:").grid(row=2, column=0, sticky=E, padx=5, pady=5)
        Label(master, text="Enter your groups:").grid(row=4, column=0, sticky=W, padx=5, pady=5)
        Label(master, text="i.e., pairfed, HFD, HFD-chow, chow", fg="red").grid(row=6, column=0, sticky=W, padx=5, pady=5)
        
        # Values of all variable fields
        self.folder_value = StringVar()
        self.folder_path = Entry(master, textvariable=self.folder_value, width=60)
        self.lights_out_default = IntVar(value=6)
        self.lights_out_am_pm_default = StringVar(value="pm")
        self.lights_out = OptionMenu(master, self.lights_out_default, *hours)
        self.lights_out_am_pm = OptionMenu(master, self.lights_out_am_pm_default, *am_pm)
        self.assign_groups_checkbox = IntVar(value=1)
        self.cb = Checkbutton(master, 
                              text="Assign Groups", 
                              variable=self.assign_groups_checkbox)
        self.groups = Entry(master, textvariable="", width=60)
        # insert our groups as default
        self.groups.delete(0, END)
        self.groups.insert(0, my_groups_default)
        
        # Position of variables on the grid
        self.select_folder_btn.grid(row=1,column=5, padx=5, pady=5)
        self.folder_path.grid(row=1, columnspan=5, sticky=W, padx=5, pady=15)
        
        self.lights_out.grid(row=2, column=1, sticky=W, padx=5, pady=5)
        self.lights_out_am_pm.grid(row=2, column=2, sticky=W, padx=5, pady=5)
        
        
        self.cb.grid(row=3, column=0, sticky=W, padx=5, pady=15)
        self.groups.grid(row=5, columnspan=5, sticky=W, padx=5, pady=5)
        
        
    def buttonbox(self):
        # add standard button box (ok and cancel)

        box = Frame(self)

        w = Button(box, text="Cancel", width=10, command=self.cancel)
        w.pack(side=LEFT, padx=5, pady=15)
        w = Button(box, text="OK", width=10, command=self.ok, default=ACTIVE)
        w.pack(side=LEFT, padx=5, pady=15)

        # same commands with keyboard (enter==ok, esc==cancel)
        self.bind("<Return>", self.ok)
        self.bind("<Escape>", self.cancel)

        box.pack()
        
        
########################################################
# Executed after clicking OK in the first popup window #
########################################################
        
    def ok(self, event=None):

        if not self.validate():
            self.initial_focus.focus_set() # put focus back
            return

        self.withdraw()
        self.update_idletasks()
        
        # retrieve user input
        self.get_input()
        # close options window
        self.cancel()
        
        # execute main functionality of the script
        self.main_function()
        print
        print("\nDone")
        try:
            self.parent.destroy()
        except:
            return
        
        
#########################################
        

    def cancel(self, event=None):

        # put focus back to the parent window
        self.parent.focus_set()
        self.destroy()

# function to validate user input in the first window
    def validate(self):
        # validate if path was given 
        
        try:
            # check in path was provided
            if len(self.folder_path.get()) == 0:
                tkMessageBox.showwarning(
                "Warning!",
                "Remember to select the path."
                )
                return 0
            else:
                return 1
        except ValueError:
            tkMessageBox.showwarning(
                "Warning!",
                "Remember to select the path."
            )
            return 0
            
# function to validate and retrieve input from the first application window
    def get_input(self):
        # executed after clicking on ok button

        # get path to folder with data
        self.main_folder_path = self.folder_path.get()
        # calculate lights out from am/pm to 24 hour clock
        # create a variable that holds the hour of lights on
        # after 12 hour of "night"
        if self.lights_out_default.get() == 12:
            if self.lights_out_am_pm_default.get()=="am":
                self.my_lights_out = 0
                self.my_lights_on = 12
            elif self.lights_out_am_pm_default.get()=="pm":
                self.my_lights_out  = self.lights_out_default.get()
                self.my_lights_on = 0
        elif self.lights_out_am_pm_default.get()=="am":
           self.my_lights_out = self.lights_out_default.get()
           self.my_lights_on = self.my_lights_out + 12
        elif self.lights_out_am_pm_default.get()=="pm":
             self.my_lights_out = self.lights_out_default.get()+12
             self.my_lights_on = self.lights_out_default.get()
    
        # check whether to assign groups
        self.assign_groups = False if self.assign_groups_checkbox.get() ==0 else True
        
        if self.assign_groups:
            self.user_gropus = self.groups.get().strip().strip(',').split(',')
        
# function to display available folders in the explorer window        
    def show_folders(self):
        # executed when select button in dialog box is clicked
        # select folder from explorer window
        self.src = tkFileDialog.askdirectory()
        self.folder_value.set(self.src)

# function to display window with user defined groups and available device ids        
    def assign_my_groups(self):
        
        # create option window
        self.option_window = Tk()
        self.option_window.title('Device Selection')
        Label(self.option_window, text="Assign available devices to your groups:").grid(row=0, columnspan=2, padx=5, pady=5)
        Label(self.option_window, text="*List of coma separated integer ids! No spaces!", fg="red").grid(row=1, columnspan=2, padx=5, pady=5)        

        # create a list of entry fields in order to read them later
        self.entries = []
        # fill all entry fields with all available devices ids
        for i in range(len(self.user_gropus)):
            # create a label for all given groups
            Label(self.option_window, text=self.user_gropus[i]).grid(row=i+2, column=0, padx=5, pady=5)
            self.grouped_devices = Entry(self.option_window, textvariable="")
            self.entries.append(self.grouped_devices)
            self.entries[i].grid(row=i+2, column=1, padx=5, pady=5)

        
        b = Button(self.option_window, text='Ok', command=self.get_assigned_gropus)
        b.grid(row=len(self.user_gropus)+3, columnspan=2, sticky='nsew', padx=20, pady=5)
        
        self.option_window.initial_focus = self.option_window
        
        self.option_window.wait_window(self.option_window)


# function to retrieve devices assigned to groups from "Device selection" window 
    def get_assigned_gropus(self):
        
        self.group_dictionary = {}
        for i in range(len(self.user_gropus)):
#            print self.user_gropus[i], self.entries[i].get()
        
            try:
                # remove leading and trailing whitespaces and comas, and split by comas
                retrieved_ids = self.entries[i].get().strip().strip(',').split(',')
                # check if all ids are integers
                self.retrieved_id_ints = [int(el) for el in retrieved_ids]
                # check if all devices were available in file
                for el in self.retrieved_id_ints:
                    if el not in self.device_indexes:
                        tkMessageBox.showwarning(
                                "Warning!",
                                "Some of the given ids might not be available in the files."
                        )
                        self.option_window.destroy()
                        # reset ids to none
                        self.retrieved_id_ints = None
                        return
            except:
                 tkMessageBox.showwarning(
                    "Warning!",
                    "List of coma separated integer ids!\nNo spaces!\n"
                )
                 self.option_window.destroy()
                 # reset ids to none
                 self.retrieved_id_ints = None
                 return
            # if it would be the first group in the dictionary
            # add ids to the dictionary
            if len(self.group_dictionary) == 0:
                # make sure that there are multiple identical ids are not assigned to the group
                self.group_dictionary[self.user_gropus[i]] = list(set(self.retrieved_id_ints))
            else:
                # check if different groups have different device ids
                for key in self.group_dictionary:
                    if bool(set(self.group_dictionary[key]).intersection(set(self.retrieved_id_ints))):
                        tkMessageBox.showwarning(
                        "Warning!",
                        "Different groups can't have identical device ids."
                            )
                        self.option_window.destroy()
                        # reset ids to none
                        self.retrieved_id_ints = None
                        return
                    
                # make sure that there are multiple identical ids are not assigned to the group
                self.group_dictionary[self.user_gropus[i]] = list(set(self.retrieved_id_ints))
        
        # destroy "Device selection" window         
        self.option_window.destroy()
        
#        for key,val in group_dictionary.items():
#            print key, ":", val
        
        # create a new dictionary from the given sequence of ids as key, 
        # and with a group as a value
        self.my_final_group_dict = {}       
        for key in self.group_dictionary:
            self.my_final_group_dict.update(dict.fromkeys(self.group_dictionary[key], key))
        # create a column with group name
        for dev_id in self.by_device_dict:
            self.by_device_dict[dev_id]['Group'] = self.by_device_dict[dev_id]['Device'].map(self.my_final_group_dict)
        
#####################################################
# END OF GUI RELATED PART
#####################################################        


#########################################################################
# My sequence of actions (reading, binning, plotting)    
########################################################################   
    def main_function(self):
        
        # read the csv files
        csv_read = self.read_csv_files()
        
        # if the read_csv_files function failed
        
        if csv_read == 0:
            return 0
            
        # read assigned groups 
        if self.assign_groups:
            self.assign_my_groups() 
            if self.retrieved_id_ints is None:
                print("Failed to assign groups")
                return
                
        # bin data
        self.bin_data()
        
        
        ######### SAVE CSV and PNG in a folder
        # create path to subfolder for results
        # MonthDay_HourMinute + the above ending + .xls
        current_time = strftime("%m%d_%H%M_%S")+"s"
        subfolder_name = "Results"+current_time
        self.subfolder_path = os.path.join(self.main_folder_path, subfolder_name)
        # if folder not yet created, create one
        if (not os.path.exists(self.subfolder_path)):
            os.mkdir(self.subfolder_path)
          
        # plot data and save png file
        self.plot()
        # save data as csv files
        self.save_data()
        
        #########################################
        
#############################################################################
        
#####################################################
# FUNCTION TO READ AND ORGANIZE EACH PIR Device DATA, 
    def read_csv_files(self):
        # reads csv files and organizes them into dataframes
        
        print("Reading your files...")
        all_dataframes = []
        # for all files in folder
        for file in os.listdir(self.main_folder_path):
            if file.endswith(".CSV"):
                # read that file into a dataframe
                file_path = os.path.join(self.main_folder_path ,file)
                # have it skip the file if 'MM:DD:YYYY hh:mm:ss' doesn't exist
#                try:
                df = pd.read_csv(file_path, parse_dates = True, index_col = 'MM:DD:YYYY hh:mm:ss')
#                print "\nBefore:\n", df.head(n=3)                  
                # strip white spaces from column names
                df.columns = df.columns.str.strip()
                
                # make day/night column. True if nighttime
                df['Nighttime'] = ~np.logical_and((df.index.hour>=self.my_lights_on),
                                                    (df.index.hour<self.my_lights_out))
                # create new columns with difference variables
                # for durations and count
                df["PIRCountDiff"] = df["PIRCount"].diff()
                df["PIRDiff"] = df["PIRDuration"].diff()  
                # make PIRBoutDur variable
                try:
                    # don't show division by zero error or nans
                    np.seterr(divide='ignore', invalid='ignore')
                    df["PIRBoutDur"] = df["PIRDiff"]/df["PIRCountDiff"]
                    # if PIRDiff is zero, the result for df["PIRDiff"]/df["PIRCountDiff"]
                    # is showing inf
                    # replace infs with zeros
                    df["PIRBoutDur"] = df["PIRBoutDur"].replace([np.inf, -np.inf], 0)
                except:
                    pass

#                print "\nAfter:\n", df.head(n=3)                 
                # add to the list of all dataframes
                all_dataframes.append(df)
                

                    
##################################################                    
        # create a single dataframe from all files
        print
        print "Number of files:", len(all_dataframes)
        print
        self.main_df = pd.concat(all_dataframes)

        # drop values of seconds seconds
        self.main_df.index = self.main_df.index.map(lambda x: x.replace(second=0))
        
        # find unique pir deviceindexes
        self.device_indexes = pd.unique(self.main_df['Device'])
        
        # split huge dataframe with all devices into 
        # single dataframe for each device
        # dictionary with dataframes for each device
        self.by_device_dict = {}
        for dev_id in self.device_indexes:
            single_df = self.main_df[self.main_df['Device'] == dev_id]
            # sort by date
            single_df = single_df.sort_index()
            self.by_device_dict[dev_id] = single_df
           
        # find first and last common dates 
        # in oredr to know how many days of data are there
        self.common_start_date = self.main_df.index.min()
        self.common_end_date = self.main_df.index.max()
        print "first", self.common_start_date
        print "last", self.common_end_date
        self.diff_all_dates = self.common_end_date - self.common_start_date
        print self.diff_all_dates.days, "days"
        # round up number of weeks
        print math.ceil(float(self.diff_all_dates.days)/7), "weeks"
        print
        
        # if less than 2 dats of data quit
        if self.diff_all_dates.days < 2:
            tkMessageBox.showwarning(
                        "Warning!",
                        "Too little data!"
                            )
            return 0
        return 1  
    
# FUNCTION TO BIN DATA INTO DESIRED BINS
    def bin_data(self):
        
        # bin days and nights separately
        # split into day and night dataframes
        # for each device there is a day and night key (1day, 1night)
        self.by_device_day_night_dict = {}
        for dev_id in self.by_device_dict:
            daytimes = self.by_device_dict[dev_id][self.by_device_dict[dev_id]['Nighttime']==False]
            nighttimes = self.by_device_dict[dev_id][self.by_device_dict[dev_id]['Nighttime']==True]
            key_name_day = str(dev_id)+"day"
            key_name_night = str(dev_id)+"night"
            self.by_device_day_night_dict[key_name_day] = daytimes
            self.by_device_day_night_dict[key_name_night] = nighttimes
        
        # if there is more than 2 weeks of data
        # bin only weekly
        if math.ceil(float(self.diff_all_dates.days)/7) > 2:
            # bin into weekly bins
            # bin only nighttime data into calendar days
            data_columns = ['PIRCountDiff','PIRDiff','PIRBoutDur']
            self.weekly_by_device_dict = {}
            for key in self.by_device_day_night_dict:
                if key.endswith('night'):
                    # if you cut out ending (day or night) in key (1night,1day) 
                    # the device id is left
                    dev_id_int = int(key[:-5])
                    # bin by calendar day: sums and means
                    weekly_sum_df = self.by_device_day_night_dict[key][data_columns].resample('7D', 
                                                            how="sum", 
                                                            closed='left', 
                                                            label='left')
                    weekly_mean_df = self.by_device_day_night_dict[key][data_columns].resample('7D', 
                                                            how="mean", 
                                                            closed='left', 
                                                            label='left')
                    # change column names
                    original_col_names = weekly_sum_df.columns.tolist()
                    col_names_sums = [name+"_sumWeekly" for name in original_col_names]
                    col_names_means = [name+"_meanWeekly" for name in original_col_names]
                    for i in range(len(original_col_names)):
                        weekly_sum_df.rename(columns={original_col_names[i]:col_names_sums[i]}, inplace=True)
                        weekly_mean_df.rename(columns={original_col_names[i]:col_names_means[i]}, inplace=True)
                    weekly_sums_and_means_df = pd.concat([weekly_sum_df,weekly_mean_df], axis=1)
                    # put back column with device id            
                    temp_df = pd.DataFrame({'Device':[dev_id_int for i in range(weekly_sums_and_means_df.shape[0])]},
                                                          index = weekly_sums_and_means_df.index)
                    with_dev_id = pd.concat([temp_df,weekly_sums_and_means_df], axis=1)
                    # if there was no data(nan), fill with zeros
#                    with_dev_id = with_dev_id.fillna(value=0)
                    # assign groups in group column
                    if self.assign_groups:
                            with_dev_id['Group'] = with_dev_id['Device'].map(self.my_final_group_dict)        
                    self.weekly_by_device_dict[dev_id_int] = with_dev_id
        ######################################################################
        # end creating self.weekly_by_device_dict
            
        # if there is less than 7 days of data
        # bin hourly and daily
        elif self.diff_all_dates.days <= 7:
            #############
            # BIN HOURLY                
            # define columns for resampling
            data_columns = ['PIRCountDiff','PIRDiff','PIRBoutDur']
            # bin into 60 minute bins
            hourly_by_device_day_night_dict = {}
            for key in self.by_device_day_night_dict:
                hourly_sum_df = self.by_device_day_night_dict[key][data_columns].resample('60T', how="sum")
                hourly_mean_df = self.by_device_day_night_dict[key][data_columns].resample('60T', how="mean")
                # change column names
                original_col_names = hourly_sum_df.columns.tolist()
                col_names_sums = [name+"_sumHourly" for name in original_col_names]
                col_names_means = [name+"_meanHourly" for name in original_col_names]
                for i in range(len(original_col_names)):
                    hourly_sum_df.rename(columns={original_col_names[i]:col_names_sums[i]}, inplace=True)
                    hourly_mean_df.rename(columns={original_col_names[i]:col_names_means[i]}, inplace=True)
                # remove daytime and nighttime NA data
                if key.endswith('day'):
                    # remove nighttime
                    hourly_sum_df['Nighttime'] = ~np.logical_and((hourly_sum_df.index.hour>=self.my_lights_on),
                                                        (hourly_sum_df.index.hour<self.my_lights_out))
                    hourly_sum_df = hourly_sum_df[hourly_sum_df['Nighttime']==False]
                    hourly_sum_df = hourly_sum_df.drop('Nighttime', axis=1)
                    hourly_mean_df['Nighttime'] = ~np.logical_and((hourly_mean_df.index.hour>=self.my_lights_on),
                                                        (hourly_mean_df.index.hour<self.my_lights_out))
                    hourly_mean_df = hourly_mean_df[hourly_mean_df['Nighttime']==False]
                    hourly_mean_df = hourly_mean_df.drop('Nighttime', axis=1)
                else:
                    # remove daytime
                    hourly_sum_df['Nighttime'] = ~np.logical_and((hourly_sum_df.index.hour>=self.my_lights_on),
                                                        (hourly_sum_df.index.hour<self.my_lights_out))
                    hourly_sum_df = hourly_sum_df[hourly_sum_df['Nighttime']==True]
                    hourly_sum_df = hourly_sum_df.drop('Nighttime', axis=1)
                    hourly_mean_df['Nighttime'] = ~np.logical_and((hourly_mean_df.index.hour>=self.my_lights_on),
                                                        (hourly_mean_df.index.hour<self.my_lights_out))
                    hourly_mean_df = hourly_mean_df[hourly_mean_df['Nighttime']==True]
                    hourly_mean_df = hourly_mean_df.drop('Nighttime', axis=1)
                hourly_sums_and_means_df = pd.concat([hourly_sum_df,hourly_mean_df], axis=1)
                hourly_by_device_day_night_dict[key] = hourly_sums_and_means_df
                
            # create a common datarframe with daytime and nighttime data
            # a list of dataframes with device ids
            df_list = []
            for key in hourly_by_device_day_night_dict:
                # if you cut out ending (day or night) in key (1night,1day) 
                # the device id is left
                if key.endswith('day'):
                    id_of_device = int(key[:-3])
                else:
                    id_of_device = int(key[:-5])
                temp_df = pd.DataFrame({'Device':[id_of_device for i in range(hourly_by_device_day_night_dict[key].shape[0])]},
                                                  index = hourly_by_device_day_night_dict[key].index)
                with_dev_id = pd.concat([temp_df,hourly_by_device_day_night_dict[key]], axis=1)
                df_list.append(with_dev_id)
                
            # back to dictionary
            self.hourly_by_device_dict = {}
            for df in df_list:
                dev_id = df['Device'][0]
                if dev_id not in self.hourly_by_device_dict:
                    self.hourly_by_device_dict[dev_id] = df
                else:
                    joined_df = pd.concat([self.hourly_by_device_dict[dev_id],df])
                    self.hourly_by_device_dict[dev_id] = joined_df.sort_index()
            # recreate nighttime column and group columns
            for device in self.hourly_by_device_dict:
                self.hourly_by_device_dict[device]['Nighttime'] = ~np.logical_and((self.hourly_by_device_dict[device].index.hour>=self.my_lights_on),
                                                        (self.hourly_by_device_dict[device].index.hour<self.my_lights_out))
                if self.assign_groups:
                    self.hourly_by_device_dict[device]['Group'] = self.hourly_by_device_dict[device]['Device'].map(self.my_final_group_dict)
            ######################################################################
            # end creating self.hourly_by_device_dict
            
            ########################
            # BIN DAILY            
            # bin into daily bins (only nighttimes)
            # self.daily_by_device_dict is created
            self.bin_daily()
            
        else: # if there is more than 7 days of data but less than 2 weeks
            # just bin into daily bins (only nighttimes)
            # self.daily_by_device_dict is created
            self.bin_daily()
            
    # function to bin nighttime data into calendar days
    # sums and means of the following columns will be calculated: 'PIRCountDiff','PIRDiff','PIRBoutDur'
    # first column will be the device id
    # last column will have the device group (if assigned)
    def bin_daily(self):
        # bin only nighttime data into calendar days
        data_columns = ['PIRCountDiff','PIRDiff','PIRBoutDur']
        self.daily_by_device_dict = {}
        for key in self.by_device_day_night_dict:
            if key.endswith('night'):
                # if you cut out ending (day or night) in key (1night,1day) 
                # the device id is left
                dev_id_int = int(key[:-5])
                # bin by calendar day: sums and means
                daily_sum_df = self.by_device_day_night_dict[key][data_columns].resample('D', how="sum")
                daily_mean_df = self.by_device_day_night_dict[key][data_columns].resample('D', how="mean")
                # change column names
                original_col_names = daily_sum_df.columns.tolist()
                col_names_sums = [name+"_sumDaily" for name in original_col_names]
                col_names_means = [name+"_meanDaily" for name in original_col_names]
                for i in range(len(original_col_names)):
                    daily_sum_df.rename(columns={original_col_names[i]:col_names_sums[i]}, inplace=True)
                    daily_mean_df.rename(columns={original_col_names[i]:col_names_means[i]}, inplace=True)
                daily_sums_and_means_df = pd.concat([daily_sum_df,daily_mean_df], axis=1)
                # put back column with device id            
                temp_df = pd.DataFrame({'Device':[dev_id_int for i in range(daily_sums_and_means_df.shape[0])]},
                                                      index = daily_sums_and_means_df.index)
                with_dev_id = pd.concat([temp_df,daily_sums_and_means_df], axis=1)
                # assign groups in group column
                if self.assign_groups:
                        with_dev_id['Group'] = with_dev_id['Device'].map(self.my_final_group_dict)        
                self.daily_by_device_dict[dev_id_int] = with_dev_id
            
        
    def plot(self):
        # if there is more than 2 weeks of data
        if math.ceil(float(self.diff_all_dates.days)/7) > 2:
            #####################            
            # PLOT WEEKLY DATA
            # create common dates
            all_earliest = []
            all_latest = []
            for device in self.weekly_by_device_dict:
                all_earliest.append(self.weekly_by_device_dict[device].index.min())
                all_latest.append(self.weekly_by_device_dict[device].index.max())
            common_dates = pd.date_range(min(all_earliest),
                                                 max(all_latest),
                                                 freq='7D')
            if self.assign_groups:
                # for each group create dataframe with common dates as indexes
                # and first column with group
                self.by_group_weekly_dict = {}  
                # for each group 
                for group in self.group_dictionary:
                    # check every device id if it is in that group
                    single_device_dfs = []
                    for key in self.weekly_by_device_dict:
                        if int(key) in self.group_dictionary[group]:
                            PIRDiff_col_name = str(key) + "PIRDiff_mean"
                            # create dataframe with common dates as indexes and PIRDiff column
                            # check if there is data for all common dates
                            dates_in_df = self.weekly_by_device_dict[key].index.tolist()
#                            print "common", common_dates
#                            print "in df", dates_in_df
                            if len(common_dates) == len(dates_in_df):
                                same = [common_dates[i]==dates_in_df[i] for i in range(len(common_dates))]
                                if all(same):   # double check that these are the same common dates
                                    temp = pd.DataFrame({PIRDiff_col_name:self.weekly_by_device_dict[key]['PIRDiff_meanWeekly'].tolist()},
                                                        index=[i+1 for i in range(len(common_dates))])
                                    single_device_dfs.append(temp)
                                else:
                                    print "Device", key, ":"
                                    print "Same amount of weeks but different week dates!"
                                    # if starting week dates are different but there is the same amount of weeks
                                    # assume the common start week days
                                    # the data for the device that had different start dates 
                                    # will be eventually plotted as week 1, 2 etc
                                    # the same as "common dates"
                                    temp = pd.DataFrame({PIRDiff_col_name:self.weekly_by_device_dict[key]['PIRDiff_meanWeekly'].tolist()},
                                                        index=[i+1 for i in range(len(common_dates))])
                                    single_device_dfs.append(temp)
                            else: # if there is different amount of weeks in data
                                # assume the common start week days
                                # the data for the device that had different start dates 
                                # will be eventually plotted as week 1, 2 etc
                                # the same as "common dates"
                                print "Device", key, ":"
                                print "Not the same amount of weeks!"
                                temp = pd.DataFrame({PIRDiff_col_name:self.weekly_by_device_dict[key]['PIRDiff_meanWeekly'].tolist()},
                                                        index=[i+1 for i in range(self.weekly_by_device_dict[key].shape[0])])
                                single_device_dfs.append(temp)
                            # concatenate groupped dfs
                            groupped_df = pd.concat(single_device_dfs, axis=1)  
                            # add groupped to the dictionary
                            self.by_group_weekly_dict[group] = groupped_df
                         
                # calculate means and standard error for the plot  
                for group in self.by_group_weekly_dict:
#                    print group
#                    print self.by_group_weekly_dict[group] 
                    # how many devices in group
                    how_many_devices = self.by_group_weekly_dict[group].shape[1]
                    # replace all Nans with zeros in order to count that into mean
#                    self.by_group_weekly_dict[group] = self.by_group_weekly_dict[group].fillna(value=0)
                    # add mean from all groupped devices column
                    self.by_group_weekly_dict[group]['PIRDiff_mean'] = self.by_group_weekly_dict[group].mean(axis=1)
    #                    print group
    #                    print self.by_group_weekly_dict[group]     
                        # add standard error column
                        # equivalent to STDEV.S/SQRT(NUMBER OF SAMPLES=unique devices) in excel
                    self.by_group_weekly_dict[group]['St_err'] = self.by_group_weekly_dict[group].iloc[:,:-1].std(axis=1)/math.sqrt(how_many_devices)
                    # name the index "week"
                    self.by_group_weekly_dict[group].index.name = 'Week'
                # PLOT
                print
                print "Plotting Weekly..."   
                fig, ax = plt.subplots(figsize=(12,6))
                for group in self.by_group_weekly_dict:
                    # plot weeks as week 1, week 2 etc...
                    weeks2plot = self.by_group_weekly_dict[group].index.values.tolist()
#                    weeks2plot = [i+1 for i in range(self.by_group_weekly_dict[group].shape[0])]
                    means = self.by_group_weekly_dict[group]['PIRDiff_mean'].tolist()
                    st_err = self.by_group_weekly_dict[group]['St_err'].tolist()
                    positive_std_err_plot = [means[i]+st_err[i] for i in range(len(st_err))]
                    negative_std_err_plot = [means[i]-st_err[i] for i in range(len(st_err))] 
                        
                    # plot means
                    means_plot = ax.plot(weeks2plot, means, alpha=1, label = group)
                    # get the color of means plot
                    recent_color = means_plot[0].get_color()
                    # fill st error shading
                    ax.fill_between(weeks2plot, positive_std_err_plot, negative_std_err_plot,
                                        color=recent_color, alpha='0.15') 
                plt.xlabel("Week", fontsize=24)
                plt.ylabel("PIRDiff", fontsize=24)
                ax.legend(loc='upper right', fontsize=8, title="Group")
                # set the max and min value of y axis
                plt.xlim(1,len(common_dates))
                # force only integer ticks
                ax.xaxis.set_major_locator(MaxNLocator(integer=True))
                plt.tight_layout()
                # save plot
                filename = "Weekly_Group_Average.png"
                path_to_save = os.path.join(self.subfolder_path,filename)
                plt.savefig(path_to_save, format='png', dpi=1000)
                print "\nSaved", os.path.join(self.subfolder_path ,filename)
                plt.close(fig)
                
            
        # if there is less than 7 days of data
        # plot hourly plot
        elif self.diff_all_dates.days <= 7:
            #####################
            # PLOT HOURLY DATA
            # create dataframe with just "PIRDiff_meanHourly" column for each device
            # and then create mean from all devices to plot
            all_means_list = []
            for key in self.hourly_by_device_dict:
                dev_id = str(key)
                new_col_with_mean = dev_id + "PIRDiff_meanHourly"
                new_df = pd.DataFrame({new_col_with_mean:self.hourly_by_device_dict[key]['PIRDiff_meanHourly']},
                                      index = self.hourly_by_device_dict[key].index) 
                all_means_list.append(new_df)
            # join all single device means into one df
            self.all_means_df = pd.concat(all_means_list, axis=1)
            
#             replace all Nans with zeros in order to count that into mean
#            self.all_means_df = self.all_means_df.fillna(value=0)
            
            # add mean from all devices column
            self.all_means_df['PIRDiff_mean'] = self.all_means_df.mean(axis=1)
            # add standard error column
            # equivalent to STDEV.S/SQRT(NUMBER OF SAMPLES=unique devices) in excel
            self.all_means_df['St_err'] = self.all_means_df.iloc[:,:-1].std(axis=1)/math.sqrt(len(self.device_indexes))
            # add nighttime column
            self.all_means_df['Nighttime'] = ~np.logical_and((self.all_means_df.index.hour>=self.my_lights_on),
                                                        (self.all_means_df.index.hour<self.my_lights_out))
            
            # PLOT
            dates2plot = self.all_means_df.index.tolist()   
            my_means = self.all_means_df['PIRDiff_mean'].tolist() 
            st_err = self.all_means_df['St_err'].tolist()
            positive_std_err_plot = [my_means[i]+st_err[i] for i in range(len(st_err))]
            negative_std_err_plot = [my_means[i]-st_err[i] for i in range(len(st_err))] 
            
            print "Plotting Hourly..."
            fig, ax = plt.subplots(figsize=(12,6))
            # plot means
            ax.plot(dates2plot, my_means, alpha=1, color = "blue") 
            # fill st error shading
            ax.fill_between(dates2plot, positive_std_err_plot, negative_std_err_plot, 
                            color="blue", alpha='0.15')  
            #day/night shading
            # how high to fill night intervals (in df there are 0 and 1 values)
            # set the max and min value of y axis
            plt.ylim(0,max(positive_std_err_plot))
            all_the_way_up = ax.get_ylim()[1]
            ax.fill_between(self.all_means_df.index, self.all_means_df["Nighttime"]*all_the_way_up, alpha=0.05, color = "black")  
            #axis titles and limits
            ax.grid(False)
            xfmt = md.DateFormatter('%m/%d')
            ax.xaxis.set_major_formatter(xfmt)
            major_hour = md.HourLocator(byhour=0)
            ax.xaxis.set_major_locator(major_hour)
            plt.xlabel("Day", fontsize=24)
            plt.ylabel("Average sec moved per minute", fontsize=24)
            plt.tick_params(axis='both', which='major', labelsize=18)
            plt.tick_params(axis='both', which='minor', labelsize=18)
            plt.tight_layout()
            # save plot
            filename = "Hourly_Average.png"
            path_to_save = os.path.join(self.subfolder_path,filename)
            plt.savefig(path_to_save, format='png', dpi=1000)
            print "\nSaved", os.path.join(self.subfolder_path ,filename)
            plt.close(fig)
            
            # PLOT DAILY
            if self.assign_groups:
                self.plot_daily()
        else: # if less than 2 weeks but more than 7 days
            if self.assign_groups:
                self.plot_daily()
            
    def plot_daily(self):
        # create common dates
        common_dates = pd.date_range(self.common_start_date.date(),
                                             self.common_end_date.date(),
                                             freq='D')
        # for each group create dataframe with common dates as indexes
        # and first column with group
        self.by_group_daily_dict = {}
        # for each group
        for group in self.group_dictionary:
            # check every device id if it is in that group
            single_device_dfs = []
            for key in self.daily_by_device_dict:
                if int(key) in self.group_dictionary[group]:
                    PIRDiff_col_name = str(key) + "PIRDiff_mean"
                    # create dataframe with common dates as indexes and PIRDiff column
                    # check if there is data for all common dates
                    dates_in_df = self.daily_by_device_dict[key].index.tolist()
                    if len(common_dates) == len(dates_in_df):
                        same = [common_dates[i]==dates_in_df[i] for i in range(len(common_dates))]
                        if all(same):   # double check that these are the same common dates
                            temp = pd.DataFrame({PIRDiff_col_name:self.daily_by_device_dict[key]['PIRDiff_meanDaily']},
                                                index=common_dates)
                            single_device_dfs.append(temp)
                        else:
                            pass
                            # the "common_dates" is the longest possible list of common dates
                            # it is not possible to have same length but different dates
                                    
                    else:
                        # if there is missing data, add nan
                        my_means = self.daily_by_device_dict[key]['PIRDiff_meanDaily'].tolist()
                        new_means = [np.nan for i in range(len(common_dates))] 
                        # transform date indexes to list 
                        # in order to get indexes for new_mean
                        common_dates_list = common_dates.tolist()
                        # insert zeros where there was no data
                        for i in range(len(dates_in_df)):
                            if dates_in_df[i] in common_dates_list:
                                # get index of that date in common
                                where_to_insert = common_dates_list.index(dates_in_df[i])
                                new_means[where_to_insert] = my_means[i]
                        temp = pd.DataFrame({PIRDiff_col_name:new_means},
                                                index=common_dates)
                        single_device_dfs.append(temp)
            # concatenate groupped dfs
            groupped_df = pd.concat(single_device_dfs, axis=1)
            # add groupped to the dictionary
            self.by_group_daily_dict[group] = groupped_df
                    
        # calculate means and standard error for the plot
        for group in self.by_group_daily_dict:
            # how many devices in group
            how_many_devices = self.by_group_daily_dict[group].shape[1]
            # replace all Nans with zeros in order to count that into mean
#            self.by_group_daily_dict[group] = self.by_group_daily_dict[group].fillna(value=0)
            # add mean from all groupped devices column
            self.by_group_daily_dict[group]['PIRDiff_mean'] = self.by_group_daily_dict[group].mean(axis=1)
            # add standard error column
            # equivalent to STDEV.S/SQRT(NUMBER OF SAMPLES=unique devices) in excel
            self.by_group_daily_dict[group]['St_err'] = self.by_group_daily_dict[group].iloc[:,:-1].std(axis=1)/math.sqrt(how_many_devices)
                
        # PLOT
        print
        print "Plotting Daily..."
        fig, ax = plt.subplots(figsize=(12,6))
        for group in self.by_group_daily_dict:
            # plot days as day 1, day 2 etc...
            days2plot = [i+1 for i in range(self.by_group_daily_dict[group].shape[0])]
            means = self.by_group_daily_dict[group]['PIRDiff_mean'].tolist()
            st_err = self.by_group_daily_dict[group]['St_err'].tolist()
            positive_std_err_plot = [means[i]+st_err[i] for i in range(len(st_err))]
            negative_std_err_plot = [means[i]-st_err[i] for i in range(len(st_err))] 
                    
            # plot means
            means_plot = ax.plot(days2plot, means, alpha=1, label = group) 
            # get the color of means plot
            recent_color = means_plot[0].get_color()
            # fill st error shading
            ax.fill_between(days2plot, positive_std_err_plot, negative_std_err_plot, 
                                    color=recent_color, alpha='0.15') 
        plt.xlabel("Day", fontsize=24)
        plt.ylabel("PIRDiff", fontsize=24)
        ax.legend(loc='upper right', fontsize=8, title="Group")
        # set the max and min value of y axis
        plt.xlim(1,len(common_dates))
        # force only integer ticks
        ax.xaxis.set_major_locator(MaxNLocator(integer=True))
        plt.tight_layout()
        # save plot
        filename = "Daily_Group_Average.png"
        path_to_save = os.path.join(self.subfolder_path,filename)
        plt.savefig(path_to_save, format='png', dpi=1000)
        print "\nSaved", os.path.join(self.subfolder_path ,filename)
        plt.close(fig)
        
    def save_data(self):
        # create subfolder for data
        data_subfolder_name = "Data"
        data_subfolder_path = os.path.join(self.subfolder_path, data_subfolder_name)
        # if folder not yet created, create one
        if (not os.path.exists(data_subfolder_path)):
            os.mkdir(data_subfolder_path)
        # write csv files
        # if there was more than 2 weeks of data
        if math.ceil(float(self.diff_all_dates.days)/7) > 2:
            for key in self.weekly_by_device_dict:
                my_csv = str(key)+"_weekly_nighttime.csv"
                # if value is nan show "NA"
                self.weekly_by_device_dict[key].to_csv(os.path.join(data_subfolder_path,my_csv), 
                                                na_rep='NA')

            if self.assign_groups:                                   
                for group in self.by_group_weekly_dict:
                    my_csv = "Group_"+group+".csv"
                    self.by_group_weekly_dict[group].to_csv(os.path.join(data_subfolder_path,my_csv), 
                                                    na_rep='NA')

        # if there was less than 1 week of data
        elif self.diff_all_dates.days <= 7:
            for key in self.hourly_by_device_dict:
                my_csv = str(key)+"_hourly.csv"
                # if value is nan show "NA"
                self.hourly_by_device_dict[key].to_csv(os.path.join(data_subfolder_path,my_csv), 
                                                na_rep='NA')
        
            # save hourly means from the plot
            my_csv = "hourly_all_means.csv"    
            self.all_means_df.to_csv(os.path.join(data_subfolder_path,my_csv), 
                                                na_rep='NA')

            # save binned daily nighttimes (one file per device)
            for key in self.daily_by_device_dict:
                my_csv = str(key)+"_daily_nighttime.csv"
                # if value is nan show "NA"
                self.daily_by_device_dict[key].to_csv(os.path.join(data_subfolder_path,my_csv), 
                                                na_rep='NA')

            if self.assign_groups:                                   
                for group in self.by_group_daily_dict:
                    my_csv = "Group_"+group+".csv"
                    self.by_group_daily_dict[group].to_csv(os.path.join(data_subfolder_path,my_csv), 
                                                    na_rep='NA')

        else: # less than 2 weeks, but more than 7 days
            # save binned daily nighttimes (one file per device)
            for key in self.daily_by_device_dict:
                my_csv = str(key)+"_daily_nighttime.csv"
                # if value is nan show "NA"
                self.daily_by_device_dict[key].to_csv(os.path.join(data_subfolder_path,my_csv), 
                                                na_rep='NA')
    
            if self.assign_groups:                                   
                for group in self.by_group_daily_dict:
                    my_csv = "Group_"+group+".csv"
                    self.by_group_daily_dict[group].to_csv(os.path.join(data_subfolder_path,my_csv), 
                                                    na_rep='NA')

#############################################################################
# CREATE GUI APPLICATION
        
root = Tkinter.Tk()
app = FedApp(root, title="Application Options")

root.mainloop()
        